#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <time.h>
#include <arpa/inet.h>
#include <ctype.h>
#include <dlfcn.h>

#include "eiwic.h"
#include "plugins.h"

EP_SETNAME("remind");

struct remind_res {
	char channel[512];
	char nick[30];
	char reason[1024];
};

void remind(void *res)
{
	struct remind_res *r;

	r=res;

	mlink->log(LOG_DEBUG, "Reminder '%s' running..", r->reason);
	mlink->plug_sendf("PRIVMSG %s :\02:: Reminder\02[%s]: %s\r\nNOTICE "
			"%s ::: Reminder: %s\r\n",
			r->channel, r->nick, r->reason, r->nick, r->reason);

	free(r);
}

unsigned long int remind_calc_time(char *str)
{
	unsigned long int t;
	
	if (*str == '+')
	{
		char *w;
		unsigned long int value;
		
		t = time(NULL);
		w = str + 1;

		while (*w > 0 && (value = (unsigned)strtol(w, &w, 10)) > 0)
		{
			fprintf(stderr,"remind: adding %d (%c)\n", value, *w);
			switch (*w)
			{
				case 'd': t += value * 60 * 60 * 24; break;
				case 'h': t += value * 60 * 60; break;
				case 'm': t += value * 60; break;
				case 's': t += value; break;
				default: return 0;
			}		
			++w;
		}				
	}
	else
	{
		char *day=NULL, *tim;
		struct tm *mt;
		int minute, hour;
		
		if ((tim = strchr(str, ',')) != NULL)
		{
			*tim++ = '\0';
			day = str;
		}

		if (strlen(tim) < 4 || strlen(tim) > 5 || 
				sscanf(tim, "%d:%d", &hour, &minute) != 2 ||
				hour > 23 || minute > 59)
			return (*(--tim) = ','?0:0);
			
		if (day != NULL && strlen(day) != 3)
			return (*(--tim) = ','?0:0);

		mt = localtime(NULL);

		mt->tm_hour = hour;
		mt->tm_min = minute;
		
		if (day)
		{			
			if (strcmp(day, "Mon") == 0)
				mt->tm_wday = 1;
			else if (strcmp(day, "Tue") == 0)
				mt->tm_wday = 2;
			else if (strcmp(day, "Wed") == 0)
				mt->tm_wday = 3;
			else if (strcmp(day, "Thu") == 0)
				mt->tm_wday = 4;
			else if (strcmp(day, "Fri") == 0)
				mt->tm_wday = 5;
			else if (strcmp(day, "Sat") == 0)
				mt->tm_wday = 6;
			else if (strcmp(day, "Sun") == 0)
				mt->tm_wday = 0;			
		}
	}
	return t;
}

int request(char *parameter, MSG *msg)
{
	char *reason;
	unsigned long int target;
	struct remind_res *res;
	
	if (parameter == NULL ||
			(reason = strchr(parameter, ' ')) == NULL ||
			strlen(reason+1) < 1)
	{		
		mlink->plug_sendf("PRIVMSG %s :Usage: !remind <date|+duration> <reason>\r\n", msg->to);
		mlink->plug_sendf("PRIVMSG %s :Examples:\r\n", msg->to);
		mlink->plug_sendf("PRIVMSG %s :-> !remind +10m30s Trigger in 10 min and 30 secs\r\n", msg->to);
		mlink->plug_sendf("PRIVMSG %s :-> !remind +1d Trigger in 1 day\r\n", msg->to);
		mlink->plug_sendf("PRIVMSG %s :-> !remind 20:15 Trigger today, 20:15\r\n", msg->to);
		mlink->plug_sendf("PRIVMSG %s :-> !remind Tue,15:30 Trigger next tuesday, 15:30\r\n", msg->to);	
		
		return -1;
	}

	*reason++ = '\0';
		
	target = remind_calc_time(parameter);

	if (target == 0)
	{
		mlink->plug_sendf("PRIVMSG %s :remind: Parse error in '%s'\r\n", msg->to,
			parameter);	
		return -1;
	}	

	mlink->plug_sendf("PRIVMSG %s :time val: %s(%li/%li) reason: %s\r\n", msg->to,
			parameter, time(NULL), target, reason);	

	if ((res = malloc(sizeof(struct remind_res))) == NULL)
	{
		mlink->log(LOG_ERROR, "malloc");
		return -1;
	}

	strncpy(res->channel, msg->to, 511);
	strncpy(res->nick, msg->nick, 29);
	strncpy(res->reason, reason, 1023);

	res->channel[511] = 0;
	res->nick[29] = 0;
	res->reason[1023] = 0;	

	mlink->plug_timer_reg(target, remind, res);

	return 1;
}

int ep_main()
{
	mlink->plug_trigger_reg("!remind", TRIG_PUBLIC, request);

	return 1;
}

